----------------------------------------------------------------------------------
-- Addon:         ProGuide - Guide Handler UI
-- Author:        Claus Gade Pedersen - Satisfyme
-- File:          ConfigMenu.lua
-- Description:   Addon config menu
----------------------------------------------------------------------------------
-- Addon version: 21
-- File Revision: 21
-- Last Edit:     2011-09-26T21:24:25Z
-- Last Edit By:  satisfyme
----------------------------------------------------------------------------------

-- Set some locals
--[[local tekcheck = LibStub("tekKonfig-Checkbox")
local tekhead  = LibStub("tekKonfig-Heading")
local tekgroup = LibStub("tekKonfig-Group")
local tekslide = LibStub("tekKonfig-Slider")]]
local AceConfig = LibStub("AceConfig-3.0")
local AceConDia = LibStub("AceConfigDialog-3.0")
local AceGUI = LibStub("AceGUI-3.0")
local devdb
local oldSliOVCScript
local elements = 0
local GAP = 8
local ldb
local L = ProGuideLocals
local GetAddOnMetadata = GetAddOnMetadata

--- Add a Config panel to the Interface Option Panel
-- @param catagoryname The name of the catagory
-- @param title The title shown in the container unit
-- @param subtitle (Optional) The subtitle shown under title
-- @param catagoryparent (Optional) If set it will show under a catagory as a sub catagory. Must match the name of the catagory to attach it to
--
local function AddConfigContainer(catagoryname, title, subtitle, catagoryparent)
  elements = elements + 1

  local panel = CreateFrame("Frame", "ProGuideConfigMenuElement" .. tostring(elements), InterfaceOptionsFramePanelContainer, "ProGuideConfigMenuTemplate")
  panel.name = catagoryname
  panel.title:SetText(title)
  if subtitle then panel.subtitle:SetText(subtitle) end
  if catagoryparent then panel.parent = catagoryparent end

  return panel
end

--- Adds a Groupe container with a border and black baclground
-- @param parent The parent frame to attach it to
-- @param headline The headline shown abow the groupe container
-- @param ... Holds one set of anchor points
--
local function AddGroupeContainer(parent, headline, ...)
  elements = elements + 1

  local group = CreateFrame("Frame", "ProGuideConfigMenuElement" .. tostring(elements), parent, "ProGuideConfigMenuGroupTemplate")
  group:SetHeadline(headline)

  if select('#',...) > 0 then group:SetPoint(...) end

  return group
end

--- Add a large text to a container
-- @param parent The parent frame to add text in
-- @param text The text shown
-- @param layer What layer the text should be shown on. default is ArtWork
-- @param ... Holds one set of anchor points
--
local function AddLargeText(parent, text, layer, ...)
  elements = elements + 1

  local FontString = parent:CreateFontString("ProGuideConfigMenuElement" .. tostring(elements), layer or "ARTWORK", "ProGuideConfigMenuLongTextTemplate")
  --local FontString = CreateFrame("FontString", "ProGuideConfigMenuElement" .. tostring(elements), parent, "ProGuideConfigMenuLongTextTemplate")
  FontString:SetText(text)
  if select('#',...) > 0 then FontString:SetPoint(...) end

  return FontString
end

--- Add a slidet to the container
-- @param parent The parent frame to attach it to
-- @param label Text shown above the slider
-- @param lowvalue The min value of the slider
-- @param highvalue The max value of the slider
-- @param ... Holds one set of anchor points
--
local function AddSlider(parent, label, lowvalue, highvalue, ...)
  elements = elements + 1

  local slider = CreateFrame("Frame", "ProGuideConfigMenuElement" .. tostring(elements), parent, "ProGuideConfigMenuSliderTemplate")

  if not oldSliOVCScript then oldSliOVCScript = slider.slider:GetScript("OnValueChanged") end

  if label then
    slider.label:SetText(label)
  end
  slider.low:SetText(lowvalue)
  slider.high:SetText(highvalue)

  -- If values is a string we expect it to be percentage else we will pharse it as a number
  if type(lowvalue) == "string" then
    slider.slider:SetMinMaxValues(tonumber((lowvalue:gsub("%%", "")))/100, tonumber((highvalue:gsub("%%", "")))/100)
    slider.slider:SetValueStep(0.05)
    slider.slider.IsPercent = true
  else
    slider.slider:SetMinMaxValues(lowvalue, highvalue)
    slider.slider:SetValueStep(1)
  end

  if select('#',...) > 0 then slider:SetPoint(...) end

  return slider
end

-- Add a checkbox to the container
local function AddCheckBox(parent, label, ...)
  elements = elements + 1

  local CheckBox = CreateFrame("CheckButton", "ProGuideConfigMenuElement" .. tostring(elements), parent, "ProGuideConfigMenuCheckButtonTemplate")
  CheckBox:SetText(label)
  CheckBox.chield = {}
  CheckBox.controlframe = nil

  if select('#',...) > 0 then CheckBox:SetPoint(...) end

  return CheckBox
end

-- ADD MAIN CONFIG MENU
local function AddMainConfigMenu()
  return {
    type = "group",
    name = L["Pro Guide"],
    args = {
      descriptionbox = {
        name = L["Description"],
        type = "group",
        inline = true,
        args = {
          description = {
            name = L["MainConfigDes"],
            type = "description",
            fontSize = "medium",
            image = "Interface\\AddOns\\ProGuide\\img\\Icon.tga",
            imageWidth = 32,
            imageHeight = 32,
          },
        },
      },
    },
  }
end

-- ADD CONFIG MENU
local function AddConfigMenu()
  return {
    type = "group",
    name = L["Config"],
    args = {
      objframe = {
        name = L["OBJFrameConfig"],
        type = "group",
        inline = true,
        args = {
          numobjframe = {
            name = L["ConfigNumOBJ"],
            desc = L["ConfigNumOBJDes"]:format(ProGuide.db.global.MaxObjFrames),
            type = "range",
            min = ProGuide.db.global.MinObjFrames,
            max = ProGuide.db.global.MaxObjFrames,
            softMin = ProGuide.db.global.MinObjFrames,
            softMax = ProGuide.db.global.MaxObjFrames,
            step = 1,
            set = function(info,val)
              ProGuide.db.profile.Config.NumObjFrames = val
              ProGuide:ShowHideObjectiveFrame()
            end,
            get = function(info) return ProGuide.db.profile.Config.NumObjFrames end,
          },
        },
      },
    },
  }
end

-- ADD GUIDE MAKERS MENU
local function AddGuideMakersMenu()
  return {
    name = L["Guide Makers"],
    type = "group",
    args = {
      GMDes = {
        name = L["GuideMakersDes"],
        type = "description",
        fontSize = "small",
        order = 1,
      },
      isGM = {
        name = L["I'm a Guide Maker"],
        desc = L["GuideMakerCheckDes"],
        type = "toggle",
        set = function(info, val)
                devdb.IsDev = val
                if devdb.IsDev and devdb.DebugFrame.Show then
                  if not ProGuide.DebugFrame:IsVisible() then ProGuide.DebugFrame:Show() end
                else
                  if ProGuide.DebugFrame:IsVisible() then ProGuide.DebugFrame:Hide() end
                end
              end,
        get = function(info) return devdb.IsDev end,
        order = 2,
      },
      DebugFrame = {
        name = L["Debug Frame"],
        type = "group",
        inline = true,
        disabled = function(info) return not devdb.IsDev end,
        args = {
          ShowDF = {
            name = L["Show Debug Frame"],
            desc = L["ShowDebugFrameDes"],
            type = "toggle",
            set = function(info, val)
                    devdb.DebugFrame.Show = val
                    if devdb.DebugFrame.Show then
                      if not ProGuide.DebugFrame:IsVisible() then ProGuide.DebugFrame:Show() end
                    else
                      if ProGuide.DebugFrame:IsVisible() then ProGuide.DebugFrame:Hide() end
                    end
                  end,
            get = function(info) return devdb.DebugFrame.Show end,
            order = 0,
          },
          DFfadeopt = {
            name = L["Debug Frame Fading"],
            type = "group",
            inline = true,
            disabled = function(info)
                         if not devdb.DebugFrame.Show or not devdb.IsDev then
                           return true
                         else
                           return false
                         end
                       end,
            order = 1,
            args = {
              fadeDF = {
                name = L["Fade Debug Frame"],
                desc = L["FadeDebugFrameDes"],
                type = "toggle",
                set = function(info, val)
                        devdb.DebugFrame.Fade = val
                        if devdb.DebugFrame.Fade then
                          ProGuide.DebugFrame:SetAlpha(devdb.DebugFrame.FadeLevel)
                        else
                          ProGuide.DebugFrame:SetAlpha(1)
                        end
                      end,
                get = function(info) return devdb.DebugFrame.Fade end,
                order = 1,
              },
              fadeDFalpha = {
                name = L["Fade Level"],
                desc = L["FadeLevelDes"],
                type = "range",
                disabled = function(info)
                             if not devdb.DebugFrame.Fade or not devdb.DebugFrame.Show or not devdb.IsDev then
                               return true
                             else
                               return false
                             end
                           end,
                min = 0,
                max = 1,
                step = .05,
                isPercent = true,
                set = function(info, val)
                        devdb.DebugFrame.FadeLevel = ProGuide:Round(val, 2)
                        if devdb.DebugFrame.Fade then
                          ProGuide.DebugFrame:SetAlpha(ProGuide:Round(val, 2))
                        end
                      end,
                get = function(info) return devdb.DebugFrame.FadeLevel end,
                order = 2
              },
            },
          },
          messages = {
            name = L["Messages"],
            type = "group",
            inline = true,
            disabled = function(info)
                         if not devdb.DebugFrame.Show or not devdb.IsDev then
                           return true
                         else
                           return false
                         end
                       end,
            order = 2,
            args = {
              errors = {
                name = L["Show Errors"],
                desc = L["ShowErrorsDes"],
                type = "toggle",
                set = function(info, val)
                  devdb.ErrorReport = val
                end,
                get = function(info) return devdb.ErrorReport end,
                order = 1,
              },
              debugmsg = {
                name = L["Show Debug Message"],
                desc = L["ShowDebugMessageDes"],
                type = "toggle",
                set = function(info, val)
                  devdb.DebugReport = val
                end,
                get = function(info) return devdb.DebugReport end,
                order = 2,
              },
              guidevalrepo = {
                name = L["Show Guide Validation Raport"],
                desc = L["ShowGuideValReportDes"],
                type = "toggle",
                set = function(info, val)
                  devdb.ValidateReport = val
                end,
                get = function(info) return devdb.ValidateReport end,
                order = 3,
              },
--@alpha@
              eventmsg = {
                name = L["Show Events"],
                desc = L["ShowEventsDes"],
                type = "toggle",
                set = function(info, val)
                  devdb.EventReport = val
                end,
                get = function(info) return devdb.EventReport end,
                order = 4,
              },
--@end-alpha@
            },
          },
        },
      },
    },
  }
end

-- ADD GUIDE LIST MENU
local function AddGuideListMenu()
  return {
    type = "group",
    name = L["Guide List"],
    args = {
      description = {
        name = L["GuideListDes"],
        type = "description",
        fontSize = "small",
        order = 0,
      },
    },
  }

  --[[local GuideList = AddConfigContainer(L["Guide List"], L["Pro Guide"] .. " - " .. L["Guide List"], L["GuideListDes"], L["Pro Guide"])
  InterfaceOptions_AddCategory(GuideList)

  return GuideList]]
end

-- ADD HELP MENU
local function AddHelpMenu()
  return {
    type = "group",
    name = L["Help"],
    args = {
      description = {
        name = L["HelpDes"],
        type = "description",
        fontSize = "small",
        order = 0,
      },
    },
  }

  --[[local Help = AddConfigContainer(L["Help"], L["Help"], L["HelpDes"], L["Pro Guide"])
  InterfaceOptions_AddCategory(Help)

  return Help]]
end

-- ADD ABOUT MENU
local function AddAboutMenu()
  local editbox = CreateFrame('EditBox', nil, UIParent)
  editbox:Hide()
  editbox:SetAutoFocus(true)
  editbox:SetHeight(32)
  editbox:SetFontObject('GameFontHighlightSmall')

  local left = editbox:CreateTexture(nil, "BACKGROUND")
  left:SetWidth(8) left:SetHeight(20)
  left:SetPoint("LEFT", -5, 0)
  left:SetTexture("Interface\\Common\\Common-Input-Border")
  left:SetTexCoord(0, 0.0625, 0, 0.625)

  local right = editbox:CreateTexture(nil, "BACKGROUND")
  right:SetWidth(8) right:SetHeight(20)
  right:SetPoint("RIGHT", 0, 0)
  right:SetTexture("Interface\\Common\\Common-Input-Border")
  right:SetTexCoord(0.9375, 1, 0, 0.625)

  local center = editbox:CreateTexture(nil, "BACKGROUND")
  center:SetHeight(20)
  center:SetPoint("RIGHT", right, "LEFT", 0, 0)
  center:SetPoint("LEFT", left, "RIGHT", 0, 0)
  center:SetTexture("Interface\\Common\\Common-Input-Border")
  center:SetTexCoord(0.0625, 0.9375, 0, 0.625)

  editbox:SetScript("OnEscapePressed", editbox.ClearFocus)
  editbox:SetScript("OnEnterPressed", editbox.ClearFocus)
  editbox:SetScript("OnEditFocusLost", editbox.Hide)
  editbox:SetScript("OnEditFocusGained", editbox.HighlightText)
  editbox:SetScript("OnTextChanged", function(self)
    self:SetText(self:GetParent().val)
    self:HighlightText()
  end)

  local fields = {"Version", "Author", "X-Category", "X-License", "X-Email", "X-Website", "X-Credits"}
  local haseditbox = {["Version"] = true, ["X-Website"] = true, ["X-Email"] = true}
  local frame = CreateFrame("Frame", nil, InterfaceOptionsFramePanelContainer)
	frame.name = L["About"]
  frame.parent = L["Pro Guide"]
  frame.addonname = "ProGuide"
	frame:Hide()
	frame:SetScript("OnShow", function(self)
    local notes = GetAddOnMetadata(self.addonname, "Notes")

    local title = self:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
    title:SetPoint("TOPLEFT", 10, -15)
    title:SetText(L["About"])

    local subtitle = self:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
    subtitle:SetHeight(32)
    subtitle:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 0, -12)
    subtitle:SetPoint("RIGHT", self, -12, 0)
    subtitle:SetNonSpaceWrap(true)
    subtitle:SetJustifyH("LEFT")
    subtitle:SetJustifyV("TOP")
    subtitle:SetText(notes)

    local anchor
    for _,field in pairs(fields) do
      local val = GetAddOnMetadata(self.addonname, field)
      if val then
        local title = self:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
        title:SetWidth(75)
        if not anchor then title:SetPoint("TOPLEFT", subtitle, "BOTTOMLEFT", -2, -8)
        else title:SetPoint("TOPLEFT", anchor, "BOTTOMLEFT", 0, -6) end
        title:SetJustifyH("RIGHT")
        title:SetText(field:gsub("X%-", ""))

        local detail = self:CreateFontString(nil, "ARTWORK", "GameFontHighlightSmall")
        detail:SetPoint("LEFT", title, "RIGHT", 4, 0)
        detail:SetPoint("RIGHT", -16, 0)
        detail:SetJustifyH("LEFT")
        detail:SetText((haseditbox[field] and "|cff9999ff" or "").. val)

        if haseditbox[field] then
          local button = CreateFrame("Button", nil, self)
          button:SetAllPoints(detail)
          button.val = val
          button:SetScript("OnClick", function(self)
            editbox:SetText(self.val)
            editbox:SetParent(self)
            editbox:SetPoint("LEFT", self)
            editbox:SetPoint("RIGHT", self)
            editbox:Show()
          end)
          button:SetScript("OnEnter", function(self)
            GameTooltip:SetOwner(self, "ANCHOR_TOPRIGHT")
            GameTooltip:SetText(L["Click and press Ctrl-C to copy"])
          end)
          button:SetScript("OnLeave", function()
            GameTooltip:Hide()
          end)
        end

        anchor = title
      end
    end

    self:SetScript("OnShow", nil)
  end)
	InterfaceOptions_AddCategory(frame)
	return frame
end

-- Initiate config menu
function ProGuide:InitConfig()
  devdb = ProGuide.db.profile.Dev

  -- Create the debug frame
  local DebugFrameUI = CreateFrame("Frame", "ProGuideDebugFrameUI", UIParent, "ProGuideDebugReportTemplate")
  ProGuide.DebugFrame = DebugFrameUI
  DebugFrameUI.db = devdb.DebugFrame
  DebugFrameUI.header:SetText(L["Debug Frame"])
  DebugFrameUI.header.tiptext = L["DebugFrameDes"]
  if DebugFrameUI.db.Show then DebugFrameUI:Show() end
  if DebugFrameUI.db.Fade then
    DebugFrameUI:SetAlpha(DebugFrameUI.db.FadeLevel)
  end

  -- Main config menu
  AceConfig:RegisterOptionsTable("PGConfig/Main", AddMainConfigMenu())
  local mainConfig = AceConDia:AddToBlizOptions("PGConfig/Main", L["Pro Guide"])

  -- Config menu
  AceConfig:RegisterOptionsTable("PGConfig/Config", AddConfigMenu())
  local Config = AceConDia:AddToBlizOptions("PGConfig/Config", L["Config"], L["Pro Guide"])

  -- Guide List
  --local GuideList = AddGuideListMenu()
  AceConfig:RegisterOptionsTable("PGConfig/GuideList", AddGuideListMenu())
  local GuideList = AceConDia:AddToBlizOptions("PGConfig/GuideList", L["Guide List"], L["Pro Guide"])

  -- Profile
  AceConfig:RegisterOptionsTable("PGConfig/Profiles", LibStub("AceDBOptions-3.0"):GetOptionsTable(ProGuide.db))
  local Profiles = AceConDia:AddToBlizOptions("PGConfig/Profiles", L["Profiles"], L["Pro Guide"])

  -- Help
  --local Help = AddHelpMenu()
  AceConfig:RegisterOptionsTable("PGConfig/Help", AddHelpMenu())
  local Help = AceConDia:AddToBlizOptions("PGConfig/Help", L["Help"], L["Pro Guide"])

  -- Guide Maker menu
  AceConfig:RegisterOptionsTable("PGConfig/GuideMakers", AddGuideMakersMenu())
  local GuideMakers = AceConDia:AddToBlizOptions("PGConfig/GuideMakers", L["Guide Makers"], L["Pro Guide"])

  -- Add About in config
  local About = AddAboutMenu()

  -----------------------------------------------------------------------------------------------------------------------------
  -- Add config button to LDB (LibDataBroker)
  -----------------------------------------------------------------------------------------------------------------------------
  LibStub:GetLibrary("LibDataBroker-1.1"):NewDataObject("ProGuide", {
		type = "launcher",
		icon = "Interface\\AddOns\\ProGuide\\img\\Icon.tga",
    -- Handle button clicks
    OnClick = function(self, button)
      if button == "LeftButton" then
        local visible = false
        if ProGuide.Obj[1]:IsVisible() then visible = true else visible = false end
        for id, _ in ipairs(ProGuide.Obj) do
          if visible then
            if ProGuide.Obj[id]:IsVisible() then
              ProGuide.Obj[id]:Hide()
            end
          else
            if id <= ProGuide.db.profile.Config.NumObjFrames then
              ProGuide.Obj[id]:Show()
            end
          end
        end
      elseif button == "RightButton" then
        InterfaceOptionsFrame_OpenToCategory(Config)
      end
    end,
    -- Add text to tooltip when showen
    OnTooltipShow = function(self)
      self:AddLine(L["Pro Guide"] .. ": " .. L["Config"],  1,1,0.5, 1)
      self:AddLine("|cff1fb3ff" .. L["IconLeftClick"],  1,1,0.5, 1)
      self:AddLine("|cff1fb3ff" .. L["IconRightClick"],  1,1,0.5, 1)
    end,
    -- Show tooltip when mouse over the button
    OnEnter = function(self)
    GameTooltip:SetOwner(self, "ANCHOR_LEFT")
    GameTooltip:SetPoint("TOPLEFT", self, "BOTTOMLEFT")
    GameTooltip:Show()
    end,
    -- Hide tooltip when mouse not over the button anymore
    OnLeave = function(self)
      GameTooltip:Hide()
    end,
  })

  InterfaceOptionsFrame_OpenToCategory(About)
end

function ProGuide:UpdateConfigDB()
  devdb = ProGuide.db.profile.Dev
  ProGuide.DebugFrame.db = ProGuide.db.profile.Dev.DebugFrame
  if ProGuide.DebugFrame.db.Show then
    if not ProGuide.DebugFrame:IsVisible() then ProGuide.DebugFrame:Show() end
  else
    if ProGuide.DebugFrame:IsVisible() then ProGuide.DebugFrame:Hide() end
  end
  if ProGuide.DebugFrame.db.Fade then
    ProGuide.DebugFrame:SetAlpha(ProGuide.DebugFrame.db.FadeLevel)
  else
    ProGuide.DebugFrame:SetAlpha(1)
  end
end