------------------------------------------------------------------------------------
-- Addon:         ProGuide - Guide Handler UI                                                 
-- Author:        Claus Gade Pedersen - Satisfyme                                 
-- File:          ErrorAndDebugging.lua
-- Description:   Handles error reports and debug reports
------------------------------------------------------------------------------------
-- Addon version: 21                                              
-- File Revision: 20                                                 
-- Last Edit:     2011-09-25T21:17:59Z                                                 
-- Last Edit By:  satisfyme                                                   
------------------------------------------------------------------------------------

local L = ProGuideLocals
local Dev = false

-- Print out the validate report
function ProGuide:ValidateReport(guidename, error)
  -- Get developer settings if not set
  if not Dev then Dev = ProGuide.db.profile.Dev end

  -- Print Validate Error if turned on
  if Dev and Dev.IsDev and Dev.DebugFrame.Show and Dev.ErrorReport then
    ProGuide:Error(L["DevFailValidate"]:format(guidename))

    -- Print Validate Report if turned on
    if Dev.ValidateReport then
      ProGuide.DebugFrame:AddMsg(L["DevValiReport"])
      for num, msg in ipairs(error) do
        ProGuide.DebugFrame:AddMsg(format("%d: %s", num, msg))
      end
      ProGuide.DebugFrame:AddMsg(L["DevValiReportEnd"])
    end
  end
end

-- Print out errors
function ProGuide:Error(error)
  -- Get developer settings if not set
  if not Dev and ProGuide.db then Dev = ProGuide.db.profile.Dev end

  -- Print Validate Error if turned on
  if not Dev then
    print(L["DevError"]:format(tostring(error)))
  elseif (Dev.IsDev and Dev.DebugFrame.Show and Dev.ErrorReport) then
    ProGuide.DebugFrame:AddMsg(L["DevError"]:format(tostring(error)))
  end
end

-- Print out debug
function ProGuide:Debug(debug)
  -- Get developer settings if not set
  if not Dev and ProGuide.db then Dev = ProGuide.db.profile.Dev end

  -- Print Validate Error if turned on
  if not Dev then
    print(L["DevDebug"]:format(tostring(debug)))
  elseif (Dev.IsDev and Dev.DebugFrame.Show and Dev.DebugReport) then
    ProGuide.DebugFrame:AddMsg(L["DevDebug"]:format(tostring(debug)))
  end
end

-- Print out events
function ProGuide:ReportEvent(event)
  -- Get developer settings if not set
  if not Dev and ProGuide.db then Dev = ProGuide.db.profile.Dev end

  -- Print Validate Error if turned on
  if not Dev then
    print(L["DevEvent"]:format(tostring(event)))
  elseif (Dev.IsDev and Dev.DebugFrame.Show and Dev.EventReport) then
    ProGuide.DebugFrame:AddMsg(L["DevEvent"]:format(tostring(event)))
  end
end

function ProGuide:UpdateDebugDB()
  Dev = ProGuide.db.profile.Dev
end