----------------------------------------------------------------------------------
-- Addon:         ProGuide - Guide Handler UI
-- Author:        Claus Gade Pedersen - Satisfyme
-- File:          ObjectiveHandler.lua
-- Description:   Handles all objectives.
----------------------------------------------------------------------------------
-- Addon version: 21
-- File Revision: 11
-- Last Edit:     2011-09-15T20:26:08Z
-- Last Edit By:  satisfyme
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
-- Local values
----------------------------------------------------------------------------------
local L = ProGuideLocals
local zonei, zonec, zonenames = {}, {}, {}
local frame, events = CreateFrame("Frame"), {}
local ItemWatchList = {}
local HashChars = {"1", "2", "3", "4", "5", "6", "7", "8", "9", "0", "a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k", "l", "m", "n", "o", "p", "q", "r", "s", "t", "u", "v", "w", "x", "y", "z", }
local Hash, HashFound
--local timer, timer025, timer1, timer5, secLeft = 0, 0, 0, 0, 0
--local fishpoles = {6256,6365,6366,6367,12225,19022,19970,25978,44050,45858,45991,45992,46337,52678}

----------------------------------------------------------------------------------
-- Local functions
----------------------------------------------------------------------------------

-- Make hash
local function MakeHashString(length, ...)
  if length >= 1 then
    return MakeHashString(length - 1, HashChars[random(1, #HashChars)], ...)
  else
    return strconcat(...)
  end
end

-- Get a DB of zones and continents
for ci,c in pairs{GetMapContinents()} do
	zonenames[ci] = {GetMapZones(ci)}
	for zi,z in pairs(zonenames[ci]) do
		zonei[z], zonec[z] = zi, ci
	end
end

-- Add waypoint arrow
local function AddWaypoint(zone, x, y, desc, c, z)
  local zi, zc = z or zone and zonei[zone], c or zone and zonec[zone]
	if not zi then
		zi, zc = GetCurrentMapZone(), GetCurrentMapContinent()
	end

	if TomTom then return TomTom:AddZWaypoint(zc, zi, x, y, desc, false) end
end

-- Set waypoint arrow
local function SetNpcArrow(info)
  if TomTom then return TomTom:SetCrazyArrow(info.uid, 1, info.Name .. " <" .. info.Type .. ">") end
end

-- Set Waypoint to a NPC
function ProGuide:SetNpcWaypoint(id)
  if id and ProGuide.Data.Npc[id] then
    if ProGuide.Data.Npc[id].uid and TomTom:IsValidWaypoint(ProGuide.Data.Npc[id].uid) then
      SetNpcArrow(ProGuide.Data.Npc[id])
    else
      ProGuide.Data.Npc[id].uid = AddWaypoint(
        ProGuide.Data.Npc[id].Loc.zone,
        ProGuide.Data.Npc[id].Loc.x,
        ProGuide.Data.Npc[id].Loc.y,
        ProGuide.Data.Npc[id].Name .. " <" .. ProGuide.Data.Npc[id].Type .. ">"
      )
    end
  end
end

function ProGuide:AddToItemWatchList(t)
  Hash = MakeHashString(10)
  HashFound = false
  while not HashFound do
    if ItemWatchList[Hash] then
      Hash = MakeHashString(10)
    else
      ItemWatchList[Hash] = t
      HashFound = true
    end
  end
  return Hash
end

function ProGuide:RemoveFromItemWatchList(hash)
  if ItemWatchList[hash] then
    ItemWatchList[hash] = nil
  end
end

----------------------------------------------------------------------------------
-- Event Handlers
----------------------------------------------------------------------------------

--
local function UpdateItemCounters()
  for hash,t in pairs(ItemWatchList) do
    if t.number then
      if t.number > GetItemCount(t.itemId, true) then
        if t.type == "itemBuyList" then
          t.frame.Counter:SetText(t.number - GetItemCount(t.itemId, true))
        else
          t.frame.Counter:SetText(L["%d/%d"]:format( GetItemCount(t.itemId, true), t.number))
        end
      else
        if t.type == "itemBuyList" then
          t.frame.Counter:Hide()
          t.frame.DoneMarker:Show()
          ProGuide:RemoveFromItemWatchList(t.ItemWatchListHash)
          t.ItemWatchListHash = nil
        else
          t.frame.Counter:Hide()
          t.frame.DoneMarker:Show()
          ProGuide:RemoveFromItemWatchList(t.ItemWatchListHash)
          t.ItemWatchListHash = nil
          ProGuide.db.char.Guides[t.guidename].Part[t.part].Step[t.id].Done = true
        end
      end
    end
  end
end

----------------------------------------------------------------------------------
-- Events
----------------------------------------------------------------------------------

-- check skill level and skill training
function events:CHAT_MSG_SKILL(msg)
  -- Check if there is a SKILL tag
  --if TourGuide.tags[TourGuide.current]:match("|SKILL|") then TGTradeS:SKILL() end

  -- Check if there is a LVLTOSKILL tag
  --if TourGuide.tags[TourGuide.current]:match("|LVLTOSKILL|") then TGTradeS:LVLTOSKILL() end
end

-- Check for new learned item recipe
function events:CHAT_MSG_SYSTEM(msg, ...)
  -- Remove profession from DB if a profession is unlearned and clear UnknownRecipe from DB so there will not be any mix ups
  --[[if msg:match("You have unlearned |cff71d5ff|Hspell:%d+|h%[(.+)%]|h|r.") then
    local ProfName = msg:match("You have unlearned |cff71d5ff|Hspell:%d+|h%[(.+)%]|h|r.")
    if ProfName == "Herb Gathering" then ProfName = "Herbalism" end
    if TGTradeS.db.char[ProfName] then TGTradeS.db.char[ProfName] = nil end
    TGTradeS.db.char.UnknownRecipe = {}
  end

  -- Check if there is a TRAINSKILL tag
  if msg:match("You have learned a new ability: (.+).") and TourGuide.tags[TourGuide.current]:match("|TRAINSKILL|") then
    TGTradeS:TRAINSKILL(msg)
  end

  -- Check if there is a LEARNITEM tag
  if msg:match("You have learned how to create a new item: ([%a:%s]+).") then
    tinsert(TGTradeS.db.char.UnknownRecipe, msg:match("You have learned how to create a new item: ([%a:%s]+)."))
    if TourGuide.tags[TourGuide.current]:match("|LEARNITEM|") then TGTradeS:LEARNITEM(msg) end
  end ]]
end

-- Check for currentcy change
function events:CURRENCY_DISPLAY_UPDATE()
  --if TourGuide.tags and TourGuide.tags[TourGuide.current]:match("|CURRENCY|") then TGTradeS:CURRENCY() end
end

-- Check for bag updates
function events:BAG_UPDATE(bId)
  -- If bag id is 0-4 then execute script
  if bId >= 0 and bId <= 4 then
    if ProGuide.IsOnline then UpdateItemCounters() end
    --UpdateItemCounters()
  end
end

-- Check for player level up
function events:PLAYER_LEVEL_UP(level)
  --if TourGuide.tags[TourGuide.current]:match("|LVL|") then TGTradeS:LVL(level) end
end

frame:SetScript("OnEvent",
  function(self, event, ...)
    events[event](self, ...)
  end
)

for k, v in pairs(events) do
  frame:RegisterEvent(k)
end