----------------------------------------------------------------------------------
-- Addon:         ProGuide - Guide Handler UI
-- Author:        Claus Gade Pedersen - Satisfyme
-- File:          ProGuide.lua
-- Description:   Main file to control Pro Guide
----------------------------------------------------------------------------------
-- Addon version: 21
-- File Revision: 20
-- Last Edit:     2011-09-25T21:17:59Z
-- Last Edit By:  satisfyme
----------------------------------------------------------------------------------

local L = ProGuideLocals
ProGuide = {}
ProGuide.Guides = {}
ProGuide.IsOnline = false
ProGuide.Obj = {}
ProGuide.Data = {
  -- Known currency to date
  ["Currency"] = {
    [61]  = L["Dalaran Jewelcrafting Token"],
    [81]  = L["Dalaran Cooking Award"],
    [241] = L["Champion's Seal"],
    [361] = L["Illustrious Jewelcrafter's Token"],
    [390] = L["Conquest Points"],
    [391] = L["Tol Barad Commendation"],
    [392] = L["Honor Points"],
    [395] = L["Justice Points"],
    [396] = L["Valor Points"],
    [402] = L["Chef's Award"],
  },
  -- Profession/Skill Names
  ["PN"] = {
    [171] = L["Alchemy"],
    [794] = L["Archaeology"],
    [164] = L["Blacksmithing"],
    [185] = L["Cooking"],
    [333] = L["Enchanting"],
    [202] = L["Engineering"],
    [129] = L["First Aid"],
    [356] = L["Fishing"],
    [182] = L["Herbalism"],
    [773] = L["Inscription"],
    [755] = L["Jewelcrafting"],
    [165] = L["Leatherworking"],
    [186] = L["Mining"],
    [393] = L["Skinning"],
    [197] = L["Tailoring"],
    [762] = L["Riding"],
  },
  -- Profession/Skill names as showen at trainder
  ["PNT"] = {
    [171] = L["Alchemist"],
    [794] = L["Archaeology"],
    [164] = L["Blacksmith"],
    [185] = L["Cook"],
    [333] = L["Enchanter"],
    [202] = L["Engineer"],
    [129] = L["First Aid"],
    [356] = L["Fishing"],
    [182] = L["Herbalist"],
    [773] = L["Scribe"],
    [755] = L["Jewelcrafter"],
    [165] = L["Leatherworker"],
    [186] = L["Miner"],
    [393] = L["Skinner"],
    [197] = L["Tailor"],
    [762] = L["Riding"],
  },
  -- Profession Level Name
  ["PLN"] = {
     [75] = L["Apprentice"],
    [150] = L["Journeyman"],
    [225] = L["Expert"],
    [300] = L["Artisan"],
    [375] = L["Master"],
    [450] = L["Grand Master"],
    [525] = L["Illustrious Grand Master"],
  },
  ["ItemType"] = {
    [171] = L["Recipe: "],
    [794] = "",
    [164] = L["Plans: "],
    [185] = L["Recipe: "],
    [333] = L["Formula: "],
    [202] = L["Schematic: "],
    [129] = L["Manual: "],
    [356] = "",
    [182] = "",
    [773] = L["Technique: "],
    [755] = L["Design: "],
    [165] = L["Pattern: "],
    [186] = "",
    [393] = "",
    [197] = L["Pattern: "],
    [762] = "",
  },
  ["Icons"] = {
    ["craft"]      = "Interface\\Icons\\Spell_Fire_FlameTounge",
    ["fly"]        = "Interface\\Icons\\Ability_Mount_Gyrocoptor",
    ["learn"]      = "Interface\\Icons\\INV_Misc_Book_08",
    ["levelskill"] = "Interface\\Icons\\Spell_Holy_DivinePurpose",
    ["none"]       = "Interface\\Icons\\INV_Misc_QuestionMark",
    ["run"]        = "Interface\\Icons\\Ability_Rogue_Sprint",
    ["teleport"]   = "Interface\\Icons\\Spell_Holy_DivineProvidence",
    ["train"]      = "Interface\\Icons\\Spell_Nature_Strength",
  },
}

local frame, events = CreateFrame("Frame"), {}

-- Fire this script when player have entered the world
function events:PLAYER_ENTERING_WORLD()
  -- Show in debug frame
  ProGuide:ReportEvent("PLAYER_ENTERING_WORLD. Guides will now be loaded and translated")

  -- Unregister this event since we don't care anymore
  self:UnregisterEvent("PLAYER_ENTERING_WORLD")

  -- Tell ProGuide that player is in the world and all data is avalible
  ProGuide.IsOnline = true

  -- Add guides that was loaded before Pro Guide is online
  ProGuide:AddPreloadedGuides()

  -- Load active guids into the Guide UI
  for i=1, ProGuide.db.profile.Config.NumObjFrames, 1 do
    ProGuide:LoadGuide(ProGuide.Obj[i], ProGuide.db.char.ObjFrames[i].Guide)
  end
end

-- Fire this script when ProGuide addon is loaded
function events:ADDON_LOADED(addon)
  if (addon == "ProGuide") then
    -- Unregister this event since we don't care anymore
    self:UnregisterEvent("ADDON_LOADED")

    -- Setup DB default values
    local defaults = {
      -- Config Settings that can be shared with other profiles
      ["profile"] = {
        ["Config"] = {
          ["NumObjFrames"] = 1,
        },
        ["Dev"] = {
          ["DebugFrame"] = {
            ["Fade"] = false,
            ["FadeLevel"] = 0.4,
            ["Show"] = false,
          },
          ["DebugReport"]    = false,
          ["ErrorReport"]    = false,
          ["EventReport"]    = false,
          ["IsDev"]          = false,
          ["ValidateReport"] = false,
        },
      },

      -- Settings that is the same for all characters
      ["global"] = {
        ["MaxObjFrames"] = 2,
        ["MinObjFrames"] = 1,
      },

      -- Character Specific DB
      ["char"] = {
        ["ObjFrames"] = {
          [1] = {
            ["Guide"] = "Tailoring 1-525",
          },
        },
        ["Guides"] = {
        },
      },
    }

    -- Setup DB
    ProGuide.db = LibStub("AceDB-3.0"):New("ProGuideDB", defaults)
    ProGuide.db.RegisterCallback(ProGuide, "OnProfileChanged", "RefreshConfig")
    ProGuide.db.RegisterCallback(ProGuide, "OnProfileCopied", "RefreshConfig")
    ProGuide.db.RegisterCallback(ProGuide, "OnProfileReset", "RefreshConfig")
--[===[@non-alpha@
    -- Make sure that we dont show events in the debug frame when not in alpha mode
    if ProGuide.db.profile.Dev.EventReport then ProGuide.db.profile.Dev.EventReport = false end
--@end-non-alpha@]===]

    -- Initiate Objective frames depending on config.
    if ProGuide.db.profile.Config.NumObjFrames > ProGuide.db.global.MaxObjFrames then
      ProGuide.db.profile.Config.NumObjFrames = ProGuide.db.global.MaxObjFrames
    end
    for i=1, ProGuide.db.global.MaxObjFrames, 1 do
      ProGuide.Obj[i] = ProGuide:ObjInitialice()
      if i > ProGuide.db.profile.Config.NumObjFrames then
        ProGuide.Obj[i]:Hide()
      end
    end

    -- Initiate Config
    ProGuide:InitConfig()

    -- Print out that the addon is loaded
    print("Pro Guide: Loaded");
  end
end

frame:SetScript("OnEvent",
  function(self, event, ...)
    events[event](self, ...)
  end
)

for k, v in pairs(events) do
  frame:RegisterEvent(k)
end

-- count table fix
local CountTableSize = {}
function ProGuide:tnum(table)
  CountTableSize = {}
  for k,v in pairs(table) do
    tinsert(CountTableSize,k)
  end
  return #CountTableSize
end

--- Round number to the closed whole number.
-- Also works for rounding decimal numbers
-- @param num The number to round
-- @param idp The number of decimal to round and show
--
function ProGuide:Round(num, idp)
  if not num then return end
  if idp and idp>0 then
    local mult = 10^idp;
    return math.floor(num * mult + 0.5) / mult;
  end
  return math.floor(num + 0.5);
end

function ProGuide:RefreshConfig()
  ProGuide:UpdateDebugDB()
  ProGuide:UpdateConfigDB()
  ProGuide:ShowHideObjectiveFrame()
  ProGuide:Debug("Profile changed!")
  print("Profile changed!")
end